//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
#define SIZE_IMAGE 180  //size in pixel

TMainForm *MainForm;


//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
  : TForm(Owner)
{

  CommPort = new TCommPort();

  StopWaitingForCommands = false;

  // Put display handles into an array
  ImagePacket[0].Display = Display0;

  // Fill displays

    ImagePacket[0].Display->Canvas->Brush->Color = (TColor)0xFFFFFF;
    ImagePacket[0].Display->Canvas->Pen->Color = ImagePacket[0].Display->Canvas->Brush->Color;
    ImagePacket[0].Display->Canvas->Rectangle(0, 0, ImagePacket[0].Display->Width, ImagePacket[0].Display->Height);


  // Put number display handles into an array
  NumberPacket[0].TB = TB0;
  NumberPacket[0].Ed = Ed0;
  NumberPacket[0].Label = Edit1;


  // Init parameters
  for (int i=0; i<NB_NUMBERS; i++) {
    NumberPacket[i].ReadFlag = false;
    NumberPacket[i].Signed = true;
    NumberPacket[i].Value = 0;
  }

}

__fastcall TMainForm::~TMainForm()
{

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ConnectClick(TObject *Sender)
{
  if (Connect->Caption == "Connect") {
    CommPort->SetCommPort(PortEdt->Text.c_str());
    CommPort->SetBaudRate(115200);
    CommPort->SetParity(NOPARITY);      // NOPARITY and friends are #defined in windows.h
    CommPort->SetByteSize(8);
    CommPort->SetStopBits(ONESTOPBIT);  // ONESTOPBIT is also from windows.h
    CommPort->OpenCommPort();
    Connect->Caption = "Disconnect";
    StopWaitingForCommands = false;
    WaitForCommands();
  } else {
    StopWaitingForCommands = true;
    Connect->Caption = "Connect";
    CommPort->CloseCommPort();
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PauseClick(TObject *Sender)
{
  if (Pause->Caption == "Pause") {
    Pause->Caption = "Resume";
  } else {
    Pause->Caption = "Pause";
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WaitForCommands(void)
{
  while (!StopWaitingForCommands) {
    ProcessCommand();
    Application->ProcessMessages();
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessCommand(void)
{
   int i;
  // Read packet header

  CommPort->ReadBytes(RxBuffer, 1);

  // Process packets
  switch (RxBuffer[0]) {
    case 0x01: { // Image packet
      unsigned char ID;
      int active_line;

      CommPort->ReadBytes(RxBuffer, 1); // Packet length
      CommPort->ReadBytes(RxBuffer, 5);

      ID = RxBuffer[0];
      if (ID>=NB_DISPLAYS) break;

      ImagePacket[ID].Type = RxBuffer[1];
      ImagePacket[ID].Height = RxBuffer[2];

      if(ImagePacket[ID].Type)
      {
        ImagePacket[ID].Width = RxBuffer[3]/2;
        ImagePacket[ID].PixNum = ImagePacket[ID].Height*ImagePacket[ID].Width*2;
      }
      else
      {
        ImagePacket[ID].Width = RxBuffer[3];
        ImagePacket[ID].PixNum = ImagePacket[ID].Height*ImagePacket[ID].Width;
      }

      active_line = (int)RxBuffer[4];




      if(ImagePacket[ID].Type)
      {
        CommPort->ReadBytes(RxBuffer, 2*ImagePacket[ID].Width);
        for (int p=0; p<ImagePacket[ID].Width*2; p++)
        {
                ImagePacket[ID].Pix[active_line*2*ImagePacket[ID].Width+p] = RxBuffer[p];
        }
      }
      else
      {
        CommPort->ReadBytes(RxBuffer, ImagePacket[ID].Width);
        for (int p=0; p<ImagePacket[ID].Width; p++)
        {
         ImagePacket[ID].Pix[active_line*ImagePacket[ID].Width+p] = RxBuffer[p];
        }
      }

     if(active_line==ImagePacket[ID].Height-1)//diplay when all arrived
       DisplayImage(ID);

      break;
    }


    case 0x03: { // Number packet (a number is comming from the robot)
      CommPort->ReadBytes(RxBuffer, 1); // Packet length

      CommPort->ReadBytes(RxBuffer, 3);

      unsigned char ID = RxBuffer[0];
      if (ID>=NB_NUMBERS) break;
      NumberPacket[ID].Signed = RxBuffer[1];
      NumberPacket[ID].Value = RxBuffer[2];
      if (!NumberPacket[ID].ReadFlag) {
        NumberPacket[ID].ReadFlag = true;
        NumberPacket[ID].TB->Enabled = false;
        NumberPacket[ID].Ed->Enabled = false;
      }
      DisplayNumber(ID);
      break;
    }
    case 0x04: { // Number packet (a number is asked by the robot)
      CommPort->ReadBytes(RxBuffer, 1); // Packet length
      CommPort->ReadBytes(RxBuffer, 1);
      unsigned char ID = RxBuffer[0];
      if (ID>=NB_NUMBERS) break;
      TxBuffer[0] = NumberPacket[ID].Value;
      if(TxBuffer[0]==-1)
        TxBuffer[0]=-2;
      CommPort->WriteBuffer(TxBuffer, 1);
      if (NumberPacket[ID].ReadFlag) {
        NumberPacket[ID].ReadFlag = false;
        NumberPacket[ID].TB->Enabled = true;
        NumberPacket[ID].Ed->Enabled = true;
      }
      break;
    }
    case 0x05: {
      break;
    }
    case 0xF0:  // Sync command
      TxBuffer[0] = 0xF0;
      CommPort->WriteBuffer(TxBuffer, 1);
      CommPort->PurgeCommPort();
      break;
    default:
      CommPort->PurgeCommPort();
      break;
  }
}

void __fastcall TMainForm::RotateImage(unsigned int ID)
{
int i;
struct TImagePacket SourceImage;
  if(ImagePacket[ID].Type)
        for (int i=0; i<ImagePacket[ID].Width*2*ImagePacket[ID].Height; i++)             //copy image
                SourceImage.Pix[i]= ImagePacket[ID].Pix[i];
  else
        for (int i=0; i<ImagePacket[ID].Width*ImagePacket[ID].Height; i++)             //copy image
                SourceImage.Pix[i]= ImagePacket[ID].Pix[i];

  SourceImage.Width = ImagePacket[ID].Width;
  SourceImage.Height = ImagePacket[ID].Height;
  ImagePacket[ID].Height = SourceImage.Width;
  ImagePacket[ID].Width = SourceImage.Height;
  if(ImagePacket[ID].Type)
  for(int xSrc = 0; xSrc < SourceImage.Width; xSrc++)
  {
        for(int ySrc = 0; ySrc < SourceImage.Height; ySrc++)
        {
        //rotate -90
        // pixel de destination :
           int xDst, yDst;
           xDst = ySrc;
           yDst = SourceImage.Width-xSrc-1;
           ImagePacket[ID].Pix[(ImagePacket[ID].Width*2*yDst+2*xDst)]=SourceImage.Pix[SourceImage.Width*2*ySrc+2*xSrc];
           ImagePacket[ID].Pix[(ImagePacket[ID].Width*2*yDst+2*xDst)+1]=SourceImage.Pix[(SourceImage.Width*2*ySrc+2*xSrc)+1];
        }
  }
  else
  for(int xSrc = 0; xSrc < SourceImage.Width; xSrc++)
  {
        for(int ySrc = 0; ySrc < SourceImage.Height; ySrc++)
        {
        //rotate -90
        // pixel de destination :
           int xDst, yDst;
           xDst = ySrc;
           yDst = SourceImage.Width-xSrc-1;
           ImagePacket[ID].Pix[ImagePacket[ID].Width*yDst+xDst]=SourceImage.Pix[SourceImage.Width*ySrc+xSrc];

        }
  }
}

//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayImage(unsigned int ID)
{

  if (Pause->Caption == "Pause") {

  unsigned int PixNum = ImagePacket[ID].PixNum;

  switch (ImagePacket[ID].Type) {
    case 0: {  // Grey-level on 1 byte
      unsigned int Pix;
      signed int Pix_s;
      int line;
      int col;
      char byte;

      if(CheckBox1->Checked)
        RotateImage(0);

      Edit2->Text = ImagePacket[ID].Height;
      Edit3->Text = ImagePacket[ID].Width;
      if(ImagePacket[ID].Height<ImagePacket[ID].Width)
                {
                ImagePacket[ID].Display->Height=(SIZE_IMAGE* ImagePacket[ID].Height)/ImagePacket[ID].Width;
                ImagePacket[ID].Display->Width=SIZE_IMAGE;
                }
      else      {
                ImagePacket[ID].Display->Width=(SIZE_IMAGE* ImagePacket[ID].Width)/ImagePacket[ID].Height;
                ImagePacket[ID].Display->Height=SIZE_IMAGE;
                }
      for (int i=0; i<PixNum; i++)
      {
        line=i/ImagePacket[ID].Width;
        col=i-line*ImagePacket[ID].Width;


        Pix= ((unsigned int)ImagePacket[ID].Pix[i])& 0xff;
        ImagePacket[ID].Display->Canvas->Brush->Color = (TColor)(Pix*65536 + Pix*256 + Pix);
        ImagePacket[ID].Display->Canvas->Pen->Color = ImagePacket[ID].Display->Canvas->Brush->Color;

        ImagePacket[ID].Display->Canvas->Rectangle(col*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,line*ImagePacket[ID].Display->Height/ImagePacket[ID].Height,
                                                (col+1)*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,  (line+1)*ImagePacket[ID].Display->Height/ImagePacket[ID].Height );
      }
    break;
    }

    case 1: {  // RGB565 image
      unsigned char Pix_1,Pix_2;
      int index;
      int Red_pix, Green_pix, Blue_pix;

      Edit2->Text = ImagePacket[ID].Height;
      Edit3->Text = ImagePacket[ID].Width;
      if(CheckBox1->Checked)
         RotateImage(0);
      if(ImagePacket[ID].Height<ImagePacket[ID].Width)
                {
                ImagePacket[ID].Display->Height=(SIZE_IMAGE* ImagePacket[ID].Height)/ImagePacket[ID].Width;
                ImagePacket[ID].Display->Width=SIZE_IMAGE;
                }
      else      {
                ImagePacket[ID].Display->Width=(SIZE_IMAGE* ImagePacket[ID].Width)/ImagePacket[ID].Height;
                ImagePacket[ID].Display->Height=SIZE_IMAGE;
                }

      for (int line=0; line<ImagePacket[ID].Height; line++)
      {
        for (int col=0; col<ImagePacket[ID].Width; col++)
        {

               // Log(line);
               // Log(col);
                ImagePacket[ID].Display->Height=(ImagePacket[ID].Display->Width* ImagePacket[ID].Height)/ImagePacket[ID].Width;

                index=line*2*ImagePacket[ID].Width+2*col;
                Pix_1 =(unsigned char)ImagePacket[ID].Pix[index];
                Pix_2 =(unsigned char)ImagePacket[ID].Pix[index+1];

                // RGB555
                Red_pix=(int)((Pix_1&0xF8));
                Green_pix=(int)(((Pix_1 & 0x07)<<5)|((Pix_2 & 0xE0)>>3));
                Blue_pix=(int)((Pix_2 & 0x1F)<<3);

                if (Red_pix > 255) Red_pix = 255;
                if (Green_pix > 255) Green_pix = 255;
                if (Blue_pix > 255) Blue_pix = 255;
                int xDst, yDst;

                        xDst = col;
                        yDst = line;


                ImagePacket[ID].Display->Canvas->Brush->Color = (TColor)(Blue_pix*65536 + Green_pix*256 +Red_pix );
                ImagePacket[ID].Display->Canvas->Pen->Color = ImagePacket[ID].Display->Canvas->Brush->Color;

                ImagePacket[ID].Display->Canvas->Rectangle(xDst*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,yDst*ImagePacket[ID].Display->Height/ImagePacket[ID].Height,
                                                    (xDst+1)*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,  (yDst+1)*ImagePacket[ID].Display->Height/ImagePacket[ID].Height) ;
        }
      }
      break;
    }
    case 2:   {  // YUV image
      unsigned char Pix_1,Pix_2;
      unsigned char RGB_image[50*50*3];
      int index_1,index_2;
      int Red_pix, Green_pix, Blue_pix;
      //extended YUV: YUV 4:2:2 -> YUV 4:4:4

      for(int line=0; line<ImagePacket[ID].Height;line++)
        for (int col=0; col<ImagePacket[ID].Width; col++)
        {
                int test;
                index_1=line*2*ImagePacket[ID].Width+2*col;
                index_2=line*3*ImagePacket[ID].Width+3*col;

                // Y component
                RGB_image[index_2]= (unsigned char)ImagePacket[ID].Pix[index_1];

                index_1++;
                index_2++;
                  test=col%2;

                if(col%2)
                {
                        // U component is in the last pix

                    RGB_image[index_2]= (unsigned char)ImagePacket[ID].Pix[index_1-2];
                    // index_1++;
                     index_2++;
                      // V component is in the actual one
                     RGB_image[index_2]= (unsigned char)ImagePacket[ID].Pix[index_1];

                } else
                {

                      // U component is in the last pix
                    RGB_image[index_2]= (unsigned char)ImagePacket[ID].Pix[index_1];
                    // index_1++;
                     index_2++;
                      // V component is in the actual one
                     RGB_image[index_2]= (unsigned char)ImagePacket[ID].Pix[index_1+2];
                }


        }
         //make an RGB image
       for(int line=0; line<ImagePacket[ID].Height;line++)
        for (int col=0; col<ImagePacket[ID].Width; col++)
        {
                float red,blue,green;
                index_2=line*3*ImagePacket[ID].Width+3*col;
               /*
                red=((float)(RGB_image[index_2]))*(64/255)-(((float)RGB_image[index_2+1])-128)*(40/255)+(((float)RGB_image[index_2+2])-128)*(88/255);
                green=((float)(RGB_image[index_2]))*(64/255)-(((float)RGB_image[index_2+1])-128)*(72/255)-(((float)RGB_image[index_2+2])-128)*(72/255);
                blue=((float)(RGB_image[index_2]))*(64/255)+(((float)RGB_image[index_2+1])-128)*(112/255)-(((float)RGB_image[index_2+2])-128)*(16/255);
               */

                red=((float)(RGB_image[index_2]))*(0.8975)-(((float)RGB_image[index_2+1]))*(-0.2086)+(((float)RGB_image[index_2+2]))*(1.1404);
                green=((float)(RGB_image[index_2]))*(1.0522)-(((float)RGB_image[index_2+1]))*(-0.2885)-(((float)RGB_image[index_2+2]))*(-0.5809);
                blue=((float)(RGB_image[index_2]))*(1.)+(((float)RGB_image[index_2+1]))*(2.0326)-(((float)RGB_image[index_2+2]))*(0.);

                if (red > 255) Red_pix = 255;
                else    Red_pix=(int)red;
                if (green > 255) Green_pix = 255;
                else    Green_pix=(int)green;
                if (blue > 255) Blue_pix = 255;
                else    Blue_pix=(int)blue;


                ImagePacket[ID].Display->Canvas->Brush->Color = (TColor)(Blue_pix*65536 + Green_pix*256 +Red_pix );
                ImagePacket[ID].Display->Canvas->Pen->Color = ImagePacket[ID].Display->Canvas->Brush->Color;

                ImagePacket[ID].Display->Canvas->Rectangle(col*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,line*ImagePacket[ID].Display->Height/ImagePacket[ID].Height,
                                                    (col+1)*ImagePacket[ID].Display->Width/ImagePacket[ID].Width,  (line+1)*ImagePacket[ID].Display->Height/ImagePacket[ID].Height );

         }
      break;

    }

    default:
      break;
  }

  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayNumber(unsigned char ID)
{
  // Only for numbers sent by the robot!
  if (Pause->Caption == "Pause") {

  if (NumberPacket[ID].ReadFlag) { // the value comes from the robot
    if (NumberPacket[ID].Signed) {
      NumberPacket[ID].TB->Position = (signed char)NumberPacket[ID].Value;
      NumberPacket[ID].Ed->Text = (signed char)NumberPacket[ID].Value;
    } else {
      NumberPacket[ID].TB->Position = (unsigned char)NumberPacket[ID].Value;
      NumberPacket[ID].Ed->Text = (unsigned char)NumberPacket[ID].Value;
    }
  }

  }

}
//---------------------------------------------------------------------------
void __fastcall TMainForm::TBChange(TObject *Sender)
{
  TTrackBar *TB;
  TB = dynamic_cast<TTrackBar*> (Sender);
  int i = TB->Tag;

  if (!NumberPacket[i].ReadFlag) { // the value will be sent to the robot
    if (NumberPacket[i].TB->Position<0) {
      NumberPacket[i].Signed = true;
      NumberPacket[i].Value = (signed char)NumberPacket[i].TB->Position;
      NumberPacket[i].Ed->Text = (signed char)NumberPacket[i].Value;
    } else {
      NumberPacket[i].Signed = false;
      NumberPacket[i].Value = (unsigned char)NumberPacket[i].TB->Position;
      NumberPacket[i].Ed->Text = (unsigned char)NumberPacket[i].Value;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EdChange(TObject *Sender)
{
  TEdit *Ed;
  Ed = dynamic_cast<TEdit*> (Sender);
  int i = Ed->Tag;

  if (!NumberPacket[i].ReadFlag) { // the value will be sent to the robot
    int val = NumberPacket[i].Ed->Text.ToInt();
    if (val<-128 || val>255) { // not in the range !
      if (NumberPacket[i].Signed) {
        NumberPacket[i].Ed->Text = (signed char)NumberPacket[i].Value;
      } else {
        NumberPacket[i].Ed->Text = (unsigned char)NumberPacket[i].Value;
      }
    } else {
      if (val<0) {
        NumberPacket[i].Signed = true;
        NumberPacket[i].TB->Position = val;
        NumberPacket[i].Value = (signed char)val;
      } else {
        NumberPacket[i].Signed = false;
        NumberPacket[i].TB->Position = val;
        NumberPacket[i].Value = (unsigned char)val;
      }
    }
  }
}
//---------------------------------------------------------------------------










